<?php
/**********************************************************************************
* Subs-ShowLocalUrlTitles.php                                                     *
***********************************************************************************
* Show Local Url Titles                                                           *
* SMF Modification by Nathaniel Baxter (nathaniel.baxter@gmail.com)               *
* =============================================================================== *
* Software version: 		Show Local Url Titles 1.2                             *
* Software by:				Nathaniel Baxter (nathaniel.baxter@gmail.com)         *
* Copyright (c) 2010, Nathaniel Baxter (nathaniel.baxter@gmail.com)               *
* All rights reserved.                                                            *
***********************************************************************************
* See the included LICENSE file for details of the license for this file and mod. *
**********************************************************************************/

if (!defined('SMF'))
	die('Hacking attempt...');

// This function is the workhorse, it gets the title for a local url.
function get_local_url_title($url)
{
	global $smcFunc, $scripturl, $context, $modSettings, $cache_titles;

	// Here is a list of the comparisons that will be made by this mod. They are in order of importance (the first entry will be compared first, and so on).
	// You can increase the number of urls that are converted to their titles, by adding entries to this array, using the format below
	//	'get_var' => array(
	//		'data' => 'db_column as url_title',
	//		'tables' => '{db_prefix}db_table',
	//		'index' => 'db_index_column',
	//		'index_type' => 'db_index_type',
	//		'where' => 'db_where',
	//		'required_vars' => array('var1' => 'val1', 'var2' => 'val2'... 'varn' => 'valn'),
	//	),
	//
	//	get_var: The $_GET variable that will be used as an index, we will search the db_index_column to match this variable.
	//	db_column: The database column that contains the relevant title data.
	//	db_table: The database table that contains the relevant information.
	//	db_index_type: (Optional) The type of the database column. If unspecified, it will be assumed to be of type string.
	//	db_where: (Optional) Any extra conditions on the query can go here.
	//	required_vars: (Optional) A list of extra $_GET variables that must be set. If a value is specified (ie. non empty string $valn) then the value of the $_GET[$varn] will be compared to $valn.
	//
	$comparisons = array(
		'msg' => array(
			'data' => 'subject as url_title',
			'tables' => '{db_prefix}messages',
			'index' => 'id_msg',
			'index_type' => 'int',
			'required_vars' => array('topic' => ''),	
		),
		'topic' => array(
			'data' => 'm.subject as url_title',
			'tables' => '{db_prefix}topics as t 
							INNER JOIN {db_prefix}messages as m ON (m.id_msg = t.id_first_msg)',
			'index' => 't.id_topic',
			'index_type' => 'int',
		),
		'board' => array(
			'data' => 'name as url_title',
			'tables' => '{db_prefix}boards',
			'index' => 'id_board',
			'index_type' => 'int',
		),
		'u' => array(
			'data' => '	member_name as url_title',
			'tables' => '{db_prefix}members',
			'index' => 'id_member',
			'index_type' => 'int',
			'required_vars' => array('action' => 'profile'),	
		),
		// Add any new comparisons below here.
	);

	// These are the operators that assign a particular value to a variable. Eg. "topic=34"
	$assignment_operators = array('=', ',');

	// These are the operators that separate two variable assignments. Eg. "topic=2;happy=true"
	$separation_operators = array(';', '/');

	// The operators signal the end of the main part of the url query string. Eg. "topic=314#msg1618"
	// Only used by the 'msg' comparison.
	$end_operators = array('#', '.');

	// Firstly strip the scripturl of 'www.' and 'http://'.
	$location = $scripturl;
	if (substr($location, 0, 7) == 'http://')
		$location = substr($location, 7);
	if (substr($location, 0, 4) == 'www.')
		$location = substr($location, 4);

	// Do we have a local url? If not, then just pass the url back.
	if (!preg_match('/(http:\/\/)(www\.)?(' . addcslashes($location, '/`\=+*{},[]^$.-!') . ')(.)*$/i' . ($context['utf8'] ? 'u' : ''), $url, $match))
		return $url;

	// We only need the part after the boardurl for the message searching.
	$after_url = substr($url, strlen($location) + 7 + (($match[2] == 'www.') ? 4 : 0));

	// Make sure that its a valid local url, otherwise just pass the url back.
	if (!in_array(substr($after_url, 0, 1), array('/', '?')))
		return $url;
	$query_string = substr($after_url, 1);

	// Got a cached title?
	if (!empty($modSettings['ShowLocalUrlTitles_caching']) && !empty($cache_titles[$query_string]))
		return $cache_titles[$query_string];

	// Get the query variables.
	$variables = parse_url_query_variables($query_string, $separation_operators, $assignment_operators);

	// Do each of the comparisons.
	foreach ($comparisons as $var => $query)
	{
		// Missing any required variables?
		if (!empty($query['required_vars']))
		{
			foreach ($query['required_vars'] as $required_var => $val)
			{
				if (!isset($variables[$required_var]) || (!empty($val) && $variables[$required_var] != $val))
					continue 2;
			}
		}

		// Check for a message title, if it makes sense to. Ie. the url is for a specific message in a topic.
		if ($var == 'msg')
		{
			$index = 0;
			// Try to match the msg number.
			foreach ($end_operators as $operator)
			{
				if (($pos = strpos($after_url, $operator . 'msg')) && ($id_msg = substr($after_url, $pos + 4)))
					$index = intval($id_msg);
			}

			// No luck? Failure then.
			if (empty($index))
				continue;
		}
		// Get the index value, if we can.
		elseif (empty($variables[$var]))
			continue;
		else
			$index = $variables[$var];

		// Sanitize any important input types!
		if (!empty($query['index_type']) && $query['index_type'] == 'int')
			$index = intval($index);

		// Get us some data!
		$request = $smcFunc['db_query']('', '
			SELECT {raw:data}
			FROM ' . $query['tables'] . '
			WHERE {raw:index} = {' . (!empty($query['index_type']) ? $query['index_type'] : 'string') . ':index_value}' . (empty($query['where']) ? '' : '
				AND {raw:where}') . '
			LIMIT 1',
			array(
				'data' => $query['data'],
				'index' => $query['index'],
				'index_value' => $index,
				'where' => !empty($query['where']) ? $query['where'] : '',
			)
		);
		$data = $smcFunc['db_fetch_assoc']($request);
		$smcFunc['db_free_result']($request);

		// If we have have title, then we're done.
		if (!empty($data) && !empty($data['url_title']))
		{
			// Do we need to cache the title?
			if (!empty($modSettings['ShowLocalUrlTitles_caching']))
				$cache_titles[$query_string] = $data['url_title'];

			return $data['url_title'];
		}
	}

	// Return the untouched url if we have no matches.
	return $url;
}

// Provide the parser functionality for the url and iurl tags respectively, used by the preparsecode function.
function get_local_url_title__callback($data)
{
	// Should we parse the url? Only do this for the unparsed_content type, or the unparsed_equals_content type, where the content and the url are the same.
	if (empty($data[1]) || substr($data[1], 1) == $data[2])
		return '[url=' . $data[2] . ']' . get_local_url_title($data[2]) . '[/url]';
	// Otherwise give back the text we were given.
	else
		return '[url' . $data[1] . ']' . $data[2] . '[/url]';
}

function get_local_iurl_title__callback($data)
{
	// Should we parse the url? Only do this for the unparsed_content type, or the unparsed_equals_content type, where the content and the url are the same.
	if (empty($data[1]) || substr($data[1], 1) == $data[2])
		return '[iurl=' . $data[2] . ']' . get_local_url_title($data[2]) . '[/iurl]';
	// Otherwise give back the text we were given.
	else
		return '[iurl' . $data[1] . ']' . $data[2] . '[/iurl]';
}

// Gets the query variables from a query string.
function parse_url_query_variables($query, $separators, $assignment_operators)
{
	$variables = array();
	$vars = array();
	// Make a list of raw variable data to process, entries should look like this "var=val".
	foreach ($separators as $separator)
	{
		if (strpos($query, $separator) !== false)
			$vars += explode($separator, $query);
	}

	// Could there be just one variable?
	if (empty($vars))
		$vars[] = $query;

	// Make the list of variables.
	foreach ($vars as $var)
	{
		foreach ($assignment_operators as $operator)
		{
			$temp = explode($operator, $var);

			if (empty($temp) || count($temp) < 2)
				continue;
			else
				$variables[$temp[0]] = $temp[1];
		}
	}
	return $variables;
}
?>